@@ -34,7 +34,7 @@ from sqlalchemy import orm
 from rattail.db import model, api
 from rattail.gpc import GPC
 from rattail.batch import BatchHandler
-from rattail.excel import ExcelReader
+from rattail.excel import ExcelReaderXLSX
 
 
 class PricingBatchHandler(BatchHandler):
@@ -100,7 +100,7 @@ class PricingBatchHandler(BatchHandler):
         Batch row data comes from input data file.
         """
         path = batch.filepath(self.config, filename=batch.input_filename)
-        reader = ExcelReader(path)
+        reader = ExcelReaderXLSX(path)
         excel_rows = reader.read_rows(progress=progress)
         session = orm.object_session(batch)
 
