@@ -405,7 +405,7 @@ class Subcommand(object):
         """
         Returns a proper User object, which the app should "run as"
         """
-        from sqlalchemy.orm.exc import NoResultFound
+        from sqlalchemy import orm
         from rattail.db.util import short_session
 
         if username is None:
@@ -418,7 +418,7 @@ class Subcommand(object):
             with short_session(session) as s:
                 try:
                     user = s.query(self.model.User).filter_by(username=username).one()
-                except NoResultFound:
+                except orm.exc.NoResultFound:
                     pass
                 else:
                     if not session:
@@ -829,6 +829,8 @@ class MakeConfig(Subcommand):
             sys.exit(1)
 
         output_path = os.path.abspath(args.output)
+        if os.path.isdir(output_path):
+            output_path = os.path.join(output_path, '{}.conf'.format(args.type))
         if os.path.exists(output_path):
             self.stderr.write("ERROR! output file already exists: {}\n".format(output_path))
             sys.exit(2)
@@ -903,7 +905,7 @@ class MakeUser(Subcommand):
         return password
 
     def mkuser_rattail(self, args):
-        from sqlalchemy.orm.exc import NoResultFound
+        from sqlalchemy import orm
         from rattail.db import auth
 
         session = self.make_session()
@@ -919,7 +921,7 @@ class MakeUser(Subcommand):
                     role = session.query(model.Role)\
                                   .filter(model.Role.name == name)\
                                   .one()
-                except NoResultFound:
+                except orm.exc.NoResultFound:
                     self.stderr.write("Role not found: {}\n".format(name))
                     session.close()
                     sys.exit(4)
@@ -1145,6 +1147,9 @@ class Upgrade(Subcommand):
     description = __doc__.strip()
 
     def add_parser_args(self, parser):
+        parser.add_argument('--system', default='rattail',
+                            help="System to which the upgrade applies.")
+
         parser.add_argument('--description',
                             help="Description for the new/matched upgrade.")
         parser.add_argument('--enabled', action='store_true', default=True,
@@ -1165,8 +1170,7 @@ class Upgrade(Subcommand):
                             "occur, but rollback (abort) the transaction at the end.")
 
     def run(self, args):
-        from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
-        from rattail.upgrades import get_upgrade_handler
+        from sqlalchemy import orm
 
         if not args.create and not args.execute:
             self.stderr.write("Must specify --create and/or --execute\n")
@@ -1178,6 +1182,7 @@ class Upgrade(Subcommand):
 
         if args.create:
             upgrade = model.Upgrade()
+            upgrade.system = args.system or 'rattail'
             upgrade.description = args.description
             upgrade.created = make_utc()
             upgrade.created_by = user
@@ -1193,12 +1198,12 @@ class Upgrade(Subcommand):
                 upgrades = upgrades.filter(model.Upgrade.description == args.description)
             try:
                 upgrade = upgrades.one()
-            except NoResultFound:
+            except orm.exc.NoResultFound:
                 self.stderr.write("no matching upgrade found\n")
                 session.rollback()
                 session.close()
                 sys.exit(1)
-            except MultipleResultsFound:
+            except orm.exc.MultipleResultsFound:
                 self.stderr.write("found {} matching upgrades\n".format(upgrades.count()))
                 session.rollback()
                 session.close()
@@ -1217,7 +1222,7 @@ class Upgrade(Subcommand):
                 sys.exit(1)
 
             # execute upgrade
-            handler = get_upgrade_handler(self.config)
+            handler = self.app.get_upgrade_handler()
             log.info("will now execute upgrade: %s", upgrade)
             if not args.dry_run:
                 handler.mark_executing(upgrade)
