@@ -78,17 +78,16 @@ class ImportSubcommand(Subcommand):
 
         if self.handler_key:
             handler = self.app.get_import_handler(self.handler_key)
-            return type(handler)
-
-        raise RuntimeError("Cannot locate a handler to use!  You "
-                           "should probably define `{}.handler_key`"
-                           "".format(self.__class__.__name__))
+            if handler:
+                return type(handler)
 
     def get_handler(self, **kwargs):
         """
         Returns a handler instance to be used by the command.
         """
         factory = self.get_handler_factory(args=kwargs.get('args'))
+        if not factory:
+            return
         kwargs.setdefault('config', getattr(self, 'config', None))
         kwargs.setdefault('command', self)
         kwargs.setdefault('progress', self.progress)
@@ -299,6 +298,11 @@ class ImportSubcommand(Subcommand):
 
     def list_all_models(self, args):
         handler = self.get_handler(args=args)
+        if not handler:
+            self.stderr.write("no handler configured!\n")
+            if self.handler_key:
+                self.stderr.write("handler key is: {}\n".format(self.handler_key))
+            sys.exit(1)
         self.stdout.write("ALL MODELS:\n")
         self.stdout.write("==============================\n")
         defaults = handler.get_default_keys()
