@@ -87,6 +87,7 @@ class DataSyncConsumer(object):
         # update all importers with current Rattail session
         for importer in six.itervalues(self.importers):
             importer.session = session
+            importer.datasync_setup()
 
     def process_changes(self, session, changes):
         """
@@ -250,17 +251,24 @@ class DataSyncImportConsumer(DataSyncConsumer):
 
     def process_changes(self, session, changes):
         """
-        Process all changes, leveraging importer(s) as much as possible.
+        Process all changes.
         """
         result = self.pre_process_changes(session, changes)
         if result is not None:
             changes = result
 
-        for change in changes:
-            self.invoke_importer(session, change)
+        self.process_changes_proper(session, changes)
 
         self.post_process_changes(session, changes)
 
+    def process_changes_proper(self, session, changes):
+        """
+        Actually process the given changes, leveraging importer(s) as
+        much as possible.
+        """
+        for change in changes:
+            self.invoke_importer(session, change)
+
     def invoke_importer(self, session, change):
         """
         For the given change, invoke the default importer behavior, if one is
