@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -30,23 +30,55 @@ import os
 import shutil
 import subprocess
 import logging
+import warnings
 
-from rattail.time import make_utc
-from rattail.util import load_object
-from rattail.mail import send_email
+from rattail.app import GenericHandler
 
 
 log = logging.getLogger(__name__)
 
 
-class UpgradeHandler(object):
+class UpgradeHandler(GenericHandler):
     """
     Base class and default implementation for upgrade handlers.
     """
 
-    def __init__(self, config):
-        self.config = config
-        self.enum = config.get_enum()
+    def get_all_systems(self, **kwargs):
+        """
+        Returns a list of all "systems" available for upgrade.
+        """
+        systems = []
+        keys = self.config.getlist('rattail.upgrades', 'systems',
+                                   default=[])
+        for key in keys:
+            label = self.config.get('rattail.upgrades',
+                                    'system.{}.label'.format(key))
+            command = self.config.get('rattail.upgrades',
+                                      'system.{}.command'.format(key))
+            systems.append({'key': key,
+                            'label': label,
+                            'command': command})
+
+        systems.sort(key=lambda s: s['label'])
+
+        systems.insert(0, {
+            'key': 'rattail',
+            'label': self.config.app_title(),
+            'command': self.config.require('rattail.upgrades', 'command'),
+        })
+
+        return systems
+
+    def get_system(self, key, require=False, **kwargs):
+        """
+        Returns the "system" record for the given key.
+        """
+        systems = self.get_all_systems(**kwargs)
+        for system in systems:
+            if system['key'] == key:
+                return system
+        if require:
+            raise KeyError("No system info found for key: {}".format(key))
 
     def executable(self, upgrade):
         """
@@ -64,19 +96,34 @@ class UpgradeHandler(object):
         upgrade.status_code = self.enum.UPGRADE_STATUS_EXECUTING
 
     def do_execute(self, upgrade, user, **kwargs):
+        """
+        Perform all steps needed to fully execute the given upgrade.
+
+        Callers should use this method; you can override
+        :meth:`execute()` to customize execution logic.
+        """
+        # execute proper
         success = self.execute(upgrade, user, **kwargs)
+
+        # declare the upgrade no longer executing
         upgrade.executing = False
+        upgrade.executed = self.app.make_utc()
+        upgrade.executed_by = user
+
+        # set upgrade status, email key
         if success:
             upgrade.status_code = self.enum.UPGRADE_STATUS_SUCCEEDED
             email_key = 'upgrade_success'
         else:
             upgrade.status_code = self.enum.UPGRADE_STATUS_FAILED
             email_key = 'upgrade_failure'
-        upgrade.executed = make_utc()
-        upgrade.executed_by = user
+
+        # send appropriate email
+        system = self.get_system(upgrade.system or 'rattail')
         url = self.config.get('tailbone', 'url.upgrade', default='#')
-        send_email(self.config, email_key, {
+        self.app.send_email(email_key, {
             'upgrade': upgrade,
+            'system_title': system['label'],
             'upgrade_url': url.format(uuid=upgrade.uuid),
         })
 
@@ -84,22 +131,41 @@ class UpgradeHandler(object):
         """
         Execute the given upgrade, as the given user.
         """
-        before_path = self.config.upgrade_filepath(upgrade.uuid, filename='requirements.before.txt', makedirs=True)
+        # record pre-upgrade status
+        before_path = self.config.upgrade_filepath(upgrade.uuid,
+                                                   filename='requirements.before.txt',
+                                                   makedirs=True)
         self.record_requirements_snapshot(before_path)
 
-        stdout_path = self.config.upgrade_filepath(upgrade.uuid, filename='stdout.log')
-        stderr_path = self.config.upgrade_filepath(upgrade.uuid, filename='stderr.log')
-        cmd = self.config.upgrade_command()
+        # get stdout/stderr file paths
+        stdout_path = self.config.upgrade_filepath(upgrade.uuid,
+                                                   filename='stdout.log')
+        stderr_path = self.config.upgrade_filepath(upgrade.uuid,
+                                                   filename='stderr.log')
+
+        # figure out the upgrade command
+        if upgrade.system:
+            system = self.get_system(upgrade.system, require=True)
+            cmd = system['command']
+            if not cmd:
+                raise ValueError("No command defined for system: {}".format(upgrade.system))
+        else:
+            cmd = self.get_system('rattail')['command']
+        cmd = self.config.parse_list(cmd)
+
+        # run the upgrade command
         log.debug("will run upgrade command: %s", cmd)
         with open(stdout_path, 'wb') as stdout:
             with open(stderr_path, 'wb') as stderr:
                 upgrade.exit_code = subprocess.call(cmd, stdout=stdout, stderr=stderr)
-
         logger = log.warning if upgrade.exit_code != 0 else log.debug
         logger("upgrade command exit code was: %s", upgrade.exit_code)
 
+        # record post-upgrade status
         after_path = self.config.upgrade_filepath(upgrade.uuid, filename='requirements.after.txt')
         self.record_requirements_snapshot(after_path)
+
+        # success as boolean
         return upgrade.exit_code == 0
 
     def record_requirements_snapshot(self, path):
@@ -136,9 +202,9 @@ def get_upgrade_handler(config, default=None):
     """
     Returns an upgrade handler object.
     """
-    spec = config.get('rattail.upgrades', 'handler', default=default)
-    if spec:
-        handler = load_object(spec)(config)
-    else:
-        handler = UpgradeHandler(config)
-    return handler
+    warnings.warn("get_upgrade_handler() function is deprecated; please "
+                  "use AppHandler.get_upgrade_handler() instead",
+                  DeprecationWarning, stacklevel=2)
+
+    app = config.get_app()
+    return app.get_upgrade_handler(default=default)
